import os
import time
from stat import *
import string,re
import ldap
import clump.db
from ldif import CreateLDIF
from registry import register
from clump.logfile import log

# Recursively descend a directory tree adding all the
# files and directories to a list
def walktree(dir,dict):
    for f in os.listdir(dir):
        if dir!=".":
            pathname = '%s/%s' % (dir, f)
        else:
            pathname = '%s' % f
        mode = os.stat(pathname)[ST_MODE]
        if S_ISDIR(mode):
            dict[pathname]=2
            # It's a directory, recurse into it
            walktree(pathname,dict)
        else:
            dict[pathname]=1

def writeObject(obj,dict,cmds):
    path=string.split(
        ldap.dn2ufn(string.replace(obj[0],clump.db.db.base,"")),",")[:-2]
    path.reverse()
    for i in range(0,len(path)-1):
        path[i]="%s.d" % string.replace(path[i],"/","_")
    path[len(path)-1]=string.replace(path[len(path)-1],"/","_")
    dir=string.join(path[:-1],"/") # Object directory
    filename=string.join(path,"/") # Object filename
    # Reset the directory entry to zero and if this generates an exception
    # then use this to create the CVS add command for that directory.
    # We cannot delete the directory since there may be other entries.
    try:
        if dir and dict[dir]:
            dict[dir]=0
    except KeyError:
        os.makedirs(dir)
        cmds.append("cvs add %s" % dir)
        dict[dir]=0   # Add dictionary entry to stop multiple creations
    # Delete the file entry. If we get an exception then generate a CVS add
    # command
    try:
        del dict[filename]
    except KeyError:
        cmds.append("cvs add %s" % filename)
    try:
        file=open(filename,"w")
        file.write(CreateLDIF(obj[0],obj[1]))
        file.close()
    except IOError,msg:
        print "** I/O Error ** Unable to write file %s: %s" % (filename,msg)

class Store:
    def __init__(self):
        self.name="store"

    def help(self):
        print ("\nSyntax : STORE <cluster>\n"+
               "\n"+
               "Stores the given cluster's configuration in a directory\n"+
               "structure pointed to by the CVS base parameter.\n")
        print

    def execute(self,line):
        if line=="":
            print "** SYNTAX ERROR ** Correct syntax is: STORE <cluster>"
            return
        top=clump.db.db.cvsbase+"/"+line
        try:
            os.chdir(top)
        except OSError:
            print "** NO STORE ** Unable to chdir to cluster store, "+\
                  "execute INIT_STORE first."
            log.write("CVS store update for cluster '%s' failed" % line)
            return
        commands=[]
        filedict={}
        walktree(".",filedict)
        # Now identify all the CVS directories and remove them because
        # we don't want to include them in our update checks!
        reCVSdir=re.compile("(^|/)CVS(/|$)")
        for key in filedict.keys():
            if reCVSdir.search(key):
                del filedict[key]
        objs=[clump.db.db.getObject("cn=%s,ou=Clusters,%s" %
                                    (line,clump.db.db.base))]
        while objs!=[]:
            newobjs=[]
            for obj in objs:
                writeObject(obj,filedict,commands)
                newobjs=newobjs+clump.db.db.getFolder(obj[0])
            objs=newobjs
        # Now loop over the remaining files to see if there are any to
        # delete. We need to do this in reverse order to ensure that
        # any directories are deleted after their contents!
        keys=filedict.keys()
        keys.sort()
        keys.reverse()
        for key in keys:
            # Don't delete directories here since we need them to
            # exist to delete files when CVS commit is called
            if filedict[key]==2:
                commands.append("cvs rm %s" % key)
            elif filedict[key]==1:
                commands.append("rm -f %s" % key)
                commands.append("cvs rm %s" % key)
        print "Updating CVS repository, please wait..."
        # Finally add the commit command
        commands.append('cvs commit -m "LDAP dump at %s"' % time.strftime(
            "%Y-%m-%d %H:%M:%S",time.localtime(time.time())))
        for cmd in commands:
            print cmd
#            if os.system(cmd+" 2> /dev/null 1> /dev/null"):
            if os.system(cmd):
                print "** CVS ERROR ** "+\
                      "Executing command '%s' failed: aborting" % cmd
                log.write("CVS store update for cluster '%s' failed" % line)
                return
        log.write("CVS store update for cluster '%s' updated successfully"
                  % line)
        # No finally delete the unneccessary directories
        for key in keys:
            if filedict[key]==2:
                os.system("rm -fr %s" % key)
        return

register(Store())
