import os
import re,string,sys
import ConfigParser
import ldap
from logfile import log
from obj.defaults import readDefaults
from obj.registry import objtype

db=None
prog=None

ClumpConfigError="CLuMP Configuration Error"

class Connection:
    def __init__(self,cfgfile="/etc/clump.conf"):
        self.db=None
        self.cfgfile=cfgfile
        self.server="localhost"
        self.hostcluster=""
        self.base=""
        self.currentDN=""
        self.logfilename="clump.log"
        self.clumpdn=""
        self.clumppw=""
        self.cvsbase="/tmp/clump"
        self.cvsroot=""
        self.open()

    def cluster(self):
        try:
            value=re.compile("cn=([^,]+),ou=Clusters").\
                   search(self.currentDN).group(1)
        except:
            value=""
        return value

    def open(self):
        data=ConfigParser.ConfigParser()
        data.read(self.cfgfile)
        self.currentDN=self.base
        try:
            log.open(data.get("clump","logfile"))
        except ConfigParser.NoOptionError:
            log.open(logfilename)
        # If there is no clump section then something is very wrong
        except ConfigParser.NoSectionError:
            raise ClumpConfigError,"No clump section found in file %s" % \
                  self.cfgfile
        try:
            self.server=data.get("clump","server")
        except ConfigParser.NoOptionError:
            raise ClumpConfigError,"No server specified"
        try:
            self.base=data.get("clump","base")
        except ConfigParser.NoOptionError:
            print "** WARNING ** No LDAP base specified"
            self.base=""
        self.currentDN=self.base
        try:
            self.clumpdn=data.get("clump","clumpdn")
        except ConfigParser.NoOptionError:
            self.clumpdn=""
        try:
            self.clumppw=data.get("clump","clumppw")
        except ConfigParser.NoOptionError:
            self.clumppw=""
        try:
            self.cvsbase=data.get("clump","cvsbase")
        except ConfigParser.NoOptionError:
            self.cvsbase=""
        try:
            self.cvsroot=data.get("clump","cvsroot")
        except ConfigParser.NoOptionError:
            self.cvsroot=""
        try:
            self.db=ldap.open(self.server)
        except ldap.LDAPError,msg:
            print "** LDAP ERROR ** %s" % msg[1]
            sys.exit(1)
        self.db.simple_bind_s(self.clumpdn,self.clumppw)
        if self.cvsroot:
            os.putenv('CVSROOT',self.cvsroot)
        if "cluster" in data.options("clump"):
            self.changedn(">%s" % data.get("clump","cluster"))
            self.hostcluster=data.get("clump","cluster")
#        log.write("Established LDAP connection using config file %s" %
#                  self.cfgfile)
        log.echo(1)
        readDefaults(data)
        return

    def modObject(self,obj):
        dn=obj[0]
        modlist=[]
        for attr in obj[1].keys():
            if re.compile("\+$").search(attr):
                modlist.append((ldap.MOD_ADD,attr[:-1],obj[1][attr]))
            elif re.compile("-$").search(attr):
                modlist.append((ldap.MOD_DELETE,attr[:-1],obj[1][attr]))
            else:
                modlist.append((ldap.MOD_REPLACE,attr,obj[1][attr]))
        self.db.modify_s(dn,modlist)

    def getObject(self,dn=None):
        if dn==None:
            dn=self.currentDN
        try:
            result=self.db.search_s(dn,ldap.SCOPE_BASE,"objectClass=*")
        except ldap.NO_SUCH_OBJECT:
            return None
        return result[0]

    def getFolder(self,dn=None,attrlist=None):
        if not dn:
            dn=self.currentDN
        try:
            result=self.db.search_s(dn,ldap.SCOPE_ONELEVEL,
                                    "objectClass=*",attrlist)
        except ldap.NO_SUCH_OBJECT:
            return []
        return result

    def addObject(self,obj):
        dn=obj[0]
        modlist=[]
        for attr in obj[1].keys():
            modlist.append((attr,obj[1][attr]))
        self.db.add_s(dn,modlist)

    def delObject(self,dn,recurse=0):
        if recurse:
            list=[]
            for obj in self.db.search_s(dn,ldap.SCOPE_SUBTREE,
                                        "objectClass=*",None):
                list.append(obj[0])
            list.reverse()
            for item in list:
                self.db.delete_s(item)
        else:
            self.db.delete_s(dn)

    def changedn(self,newdn):
        self.currentDN=self.string2dn(newdn)
        return

    def string2dn(self,text,create=0):
        dirs=string.split(text,">")
        if dirs[0]=="":
            dn="ou=Clusters,%s" % self.base
            del dirs[0]
        else:
            dn=self.currentDN
        i=0
        for dir in dirs:
            i=i+1
            objs=self.getFolder(dn)
            found=0
            for obj in objs:
                if string.lower(string.split(ldap.dn2ufn(obj[0]),",")[0]) \
                                == string.lower(dir):
                    dn=obj[0]
                    found=1
                    break
            if found==0:
                # If this is the last directory entry and we are creating
                # an object then we need to add the last part of the DN
                # by hand since there is no entry in the LDAP server
                if i==len(dirs) and create==1:
                    try:
                        dn="%s=%s,%s" % (objtype(self.getObject(
                            dn)[1]['objectType'][0]).namefield,dir,dn)
                    except KeyError:
                        print "** NOT A DIRECTORY ** "+\
                              "You can only create objects in directories"
                        return ""
                else:
                    print "** INVALID PATH ** '%s' is not a valid path" % text
                    return ""
        return dn

    def dn2string(self,dn=None):
        if dn==None:
            dn=self.currentDN
        fields=string.split(dn,",")
        fields=fields[:-len(string.split(self.base,","))-1]
        text=""
        for field in fields:
            m=re.compile("^[^=]+=(.*?)$").match(field)
            if not m:
                print "** DN INVALID ** '%s' is not a valid CLuMP dn" % dn
                return ""
            text=">%s" % m.group(1)+text
        return text
