import string
import clump.db
from libglade import *
from gnome.ui import *
from gnome.util import *
from gtk import *
from clump.obj.defaults import getDefaults

GUIError="Error with GUI interface"

class TextWidget:
    def __init__(self,widgets,args):
        self.widget=widgets[0]

    def setValue(self,value=[""]):
        self.widget.set_text(value[0])

    def getValue(self):
        return self.widget.get_text()

class TextWindowWidget:
    def __init__(self,widgets,args):
        self.widget=widgets[0]

    def setValue(self,value=[""]):
        self.widget.delete_text(0,-1)
        self.widget.insert_text(value[0])

    def getValue(self):
        return self.widget.get_chars(0,-1)

class ToggleWidget:
    def __init__(self,widgets,args):
        self.widget=widgets[0]
        self.no  = [ "0", "no",  "off", "false" ]
        self.yes = [ "1", "yes", "on",  "true"  ]

    def setValue(self,value=["0"]):
        value=string.lower(value[0])
        if value in self.yes:
            self.widget.set_active(-1)
        elif value in self.no:
            self.widget.set_active(0)

    def getValue(self):
        if self.widget.get_active():
            return self.yes[0]
        else:
            return self.no[0]

class BiStateWidget(ToggleWidget):
    def __init__(self,widgets,args):
        ToggleWidget.__init__(self,widgets,args)
        self.widget=widgets # Override Toggle widget default

    def setValue(self,value=["0"]):
        value=string.lower(value[0])
        if value in self.yes:
            self.widget[0].set_active(-1)
            self.widget[1].set_active(0)
        elif value in self.no:
            self.widget[0].set_active(0)
            self.widget[1].set_active(-1)
        
    def getValue(self):
        if self.widget[0].get_active() and not self.widget[1].get_active():
            return self.yes[0]
        elif not self.widget[0].get_active() and self.widget[1].get_active():
            return self.no[0]
        else:
            raise GUIError,"Bistate widget in impossible state."

class TriStateWidget(ToggleWidget):
    def __init__(self,widgets,args):
        ToggleWidget.__init__(self,widgets,args)
        self.widget=widgets # Override Toggle widget default

    def setValue(self,value=["0"]):
        value=string.lower(value[0])
        if value in self.yes:
            self.widget[0].set_active(-1)
            self.widget[1].set_active(0)
            self.widget[2].set_active(0)
        elif value in self.no:
            self.widget[0].set_active(0)
            self.widget[1].set_active(-1)
            self.widget[2].set_active(0)
        else:
            self.widget[0].set_active(0)
            self.widget[1].set_active(0)
            self.widget[2].set_active(-1)
        
    def getValue(self):
        if self.widget[0].get_active():
            return self.yes[0]
        elif self.widget[1].get_active():
            return self.no[0]
        else:
            return ""

class ComboWidget:
    def __init__(self,widgets,args):
        self.widget=widgets[0]
        self.list=args['list']

    def setValue(self,value=[""]):
        list=self.list()
        if list==[]:
            self.widget.set_popdown_strings([""])
        else:
            self.widget.set_popdown_strings(list)
        self.widget.entry.set_text(value[0])

    def getValue(self):
        return self.widget.entry.get_text()

class BaseGUI:
    wTree=None
    def __init__(self,keywidget,object,signals,fields,onNewDisable=[]):
        self.fields=[]
        self.keywidget=keywidget
        BaseGUI.wTree.signal_autoconnect(signals.__dict__)
        for field in fields:
            widgets=[]
            for wg in field[0]:
                widgets.append(BaseGUI.wTree.get_widget(wg))
            self.fields.append((field[1],field[2](widgets,field[3])))
        self.object=object()
        self.onNewDisable=onNewDisable
        self.dn=self.object.namefield
        self.newobject=""
        self.data=("",{})
        self.parent=None

    def setup(self,data):
        BaseGUI.wTree.get_widget(self.keywidget).set_editable(0)
        for field in self.fields:
            try:
                field[1].setValue(data[1][field[0]])
            except KeyError:
                field[1].setValue()
        self.data=data
        self.newobject=""

    def setupNew(self,base):
        dict=getDefaults(self.object.name)
        dict['objectClass']=self.object.objclass
        data=(base,dict)
        self.setup(data)
        BaseGUI.wTree.get_widget(self.keywidget).set_editable(1)
        for wname in self.onNewDisable:
            BaseGUI.wTree.get_widget(wname).set_sensitive(0)
        self.newobject=base

    def checkData(self):
        # Delete any empty lists hanging around from custom getData's
        for key in self.data[1].keys():
            if self.data[1][key]==[]:
                del self.data[1][key]
        return -1

    def getData(self):
        for field in self.fields:
            value=field[1].getValue()
            if value:
                if type(value)!=type([]):
                    self.data[1][field[0]]=[value]
                else:
                    self.data[1][field[0]]=value
        if self.checkData():
            self.object.onSet(self.data[0],self.data[1])
            return -1
        else:
            print "GETDATA FAILED: data check failed to pass"
            return 0

    def update(self):
        if not self.getData():
            return 0
        if self.newobject:
            self.data=("%s=%s,%s" % (
                self.dn,self.data[1][self.dn][0],self.newobject),self.data[1])
            if self.object.createNew(self.data[0],self.data[1]):
                self.newobject=""
                BaseGUI.wTree.get_widget(self.keywidget).set_editable(0)
                for wname in self.onNewDisable:
                    BaseGUI.wTree.get_widget(wname).set_sensitive(1)
            else:
                return 0
        else:
            clump.db.db.modObject(self.data)
        if self.parent:
            self.parent.instance.setup(self.parent.instance.data)
            self.parent=None
        return -1

if not BaseGUI.wTree:
    # Load the GUI interface from GLADE
    BaseGUI.wTree=GladeXML("/usr/lib/clump/clump.glade")

# Setup the default signals for the GUI
class DefaultSignals:
    # Do nothing handler + prevent other handlers getting event
    def NULL_handler(obj,evt):
        return -1

BaseGUI.wTree.signal_autoconnect(DefaultSignals.__dict__)

# Create common pixmaps
class Pixmap:
    path="/usr/lib/clump/pixmaps/"
    yes=GtkPixmap(BaseGUI.wTree.get_widget("CLUSTER_window"),
                  path+"yes.xpm",GdkColor(255,255,255))
    no=GtkPixmap(BaseGUI.wTree.get_widget("CLUSTER_window"),
                 path+"no.xpm",GdkColor(255,255,255))
    warning=GtkPixmap(BaseGUI.wTree.get_widget("CLUSTER_window"),
                      path+"warning.xpm",GdkColor(255,255,255))
    
