import os
import tempfile
from basegui import *
from clump.interface import *
from clump.obj.configfile import File
from gtk import *
from filechooser import chooseFile

#===========================================================================
# SIGNAL FUNCTIONS: Functions used to handle signals
#
seditors=["pico","jed","vi"]               # Screen display-only Editors

class FileSignals:
    def FILE_destroy(obj,event):
        BaseGUI.wTree.get_widget("FILE_window").hide()
        return -1

    def FILE_cancel(obj):
        BaseGUI.wTree.get_widget("FILE_window").hide()

    def FILE_apply(obj):
        FileGUI.instance.update()

    def FILE_commit(obj):
        if FileGUI.instance.update():
            BaseGUI.wTree.get_widget("FILE_window").hide()

    def FILE_choose_filename(obj):
        chooseFile("FILE_name")

    def FILE_choose_import(obj):
        chooseFile("FILE_import")

    def FILE_import_file(obj):
        fname=BaseGUI.wTree.get_widget("FILE_import").get_text()
        try:
            file=open(fname,'r')
        except:
            print "** FILE IO ERROR ** Unable to open data file"
            return
        text=file.read()
        file.close()
        widget=BaseGUI.wTree.get_widget("FILE_source")
        widget.delete_text(0,-1)
        widget.insert_text(text)

    def FILE_export_file(obj):
        widget=BaseGUI.wTree.get_widget("FILE_source")
        text=widget.get_chars(0,-1)
        try:
            editor=os.environ['EDITOR']
        except KeyError:
            editor="/usr/bin/xemacs"
        xterm=0
        for ed in seditors:
            if string.find(editor,ed)!=-1:
                xterm=1
                break
        fname=tempfile.mktemp()
        file=open(fname,"w")
        file.write(text)
        file.close()
        if xterm:
            os.system("xterm -e %s %s" % (editor,fname))
        else:
            os.system("%s %s" % (editor,fname))
        file=open(fname,"r")
        file.read(text)
        file.close()
        os.system("rm %s" % fname)
        widget.delete_text(0,-1)
        widget.insert_text(text)

#===========================================================================
# UTILITY FUNCTIONS: Used for filling in tables, combo box lists etc.
#
def getFileList(attrib):
    list=[]
    files=getAllInCluster("configFile")
    for file in files:
        try:
            if not file[1][attrib][0] in list:
                list.append(file[1][attrib][0])
        except KeyError:
            continue
    list.sort()
    return list

def getFileOwnerList():
    return getFileList("uid")

def getFileGroupList():
    return getFileList("gid")

def getFileTypeList():
    return ['plain','python','macro']

#===========================================================================
# GUI Class: Used to fill the GUI window
#
class FileGUI(BaseGUI):
    instance=None
    def __init__(self):
        BaseGUI.__init__(self,"FILE_label",File,FileSignals,[
            (["FILE_label"],     "cn",               TextWidget,   {} ),
            (["FILE_name"],      "fileName",         TextWidget,   {} ),
            (["FILE_owner"],     "uid",              ComboWidget,
             { 'list' : getFileOwnerList } ),
            (["FILE_group"],     "gid",              ComboWidget,
             { 'list' : getFileGroupList } ),
            (["FILE_type"],      "fileType",         ComboWidget,
             { 'list' : getFileTypeList } ),
            (["FILE_source"],    "sourceCode",       TextWindowWidget,   {} )
            ])
        FileGUI.instance=self

    def setupOwnerAccess(self):
        try:
            value=eval(self.data[1]['accessMode'][0][-3])
        except KeyError:
            value=0
        if (value&4):
            BaseGUI.wTree.get_widget("FILE_owner_read").set_active(1)
        else:
            BaseGUI.wTree.get_widget("FILE_owner_read").set_active(0)
        if (value&2):
            BaseGUI.wTree.get_widget("FILE_owner_write").set_active(1)
        else:
            BaseGUI.wTree.get_widget("FILE_owner_write").set_active(0)
        if (value&1):
            BaseGUI.wTree.get_widget("FILE_owner_exec").set_active(1)
        else:
            BaseGUI.wTree.get_widget("FILE_owner_exec").set_active(0)
            
    def setupGroupAccess(self):
        try:
            value=eval(self.data[1]['accessMode'][0][-2])
        except KeyError:
            value=0
        if (value&4):
            BaseGUI.wTree.get_widget("FILE_group_read").set_active(1)
        else:
            BaseGUI.wTree.get_widget("FILE_group_read").set_active(0)
        if (value&2):
            BaseGUI.wTree.get_widget("FILE_group_write").set_active(1)
        else:
            BaseGUI.wTree.get_widget("FILE_group_write").set_active(0)
        if (value&1):
            BaseGUI.wTree.get_widget("FILE_group_exec").set_active(1)
        else:
            BaseGUI.wTree.get_widget("FILE_group_exec").set_active(0)
            
    def setupAllAccess(self):
        try:
            value=eval(self.data[1]['accessMode'][0][-1])
        except KeyError:
            value=0
        if (value&4):
            BaseGUI.wTree.get_widget("FILE_all_read").set_active(1)
        else:
            BaseGUI.wTree.get_widget("FILE_all_read").set_active(0)
        if (value&2):
            BaseGUI.wTree.get_widget("FILE_all_write").set_active(1)
        else:
            BaseGUI.wTree.get_widget("FILE_all_write").set_active(0)
        if (value&1):
            BaseGUI.wTree.get_widget("FILE_all_exec").set_active(1)
        else:
            BaseGUI.wTree.get_widget("FILE_all_exec").set_active(0)

    def setupSetIDBits(self):
        try:
            value=eval(self.data[1]['accessMode'][0][-4])
        except KeyError:
            value=0
        except IndexError:
            value=0
        if (value&4):
            BaseGUI.wTree.get_widget("FILE_owner_setuid").set_active(1)
        else:
            BaseGUI.wTree.get_widget("FILE_owner_setuid").set_active(0)
        if (value&2):
            BaseGUI.wTree.get_widget("FILE_group_setgid").set_active(1)
        else:
            BaseGUI.wTree.get_widget("FILE_group_setgid").set_active(0)
        if (value&1):
            BaseGUI.wTree.get_widget("FILE_all_text").set_active(1)
        else:
            BaseGUI.wTree.get_widget("FILE_all_text").set_active(0)
            
    def setup(self,data):
        BaseGUI.setup(self,data)
        self.setupOwnerAccess()
        self.setupGroupAccess()
        self.setupAllAccess()
        self.setupSetIDBits()

    def getOwnerAccess(self):
        value=0
        if BaseGUI.wTree.get_widget("FILE_owner_read").get_active():
            value=value|4
        if BaseGUI.wTree.get_widget("FILE_owner_write").get_active():
            value=value|2
        if BaseGUI.wTree.get_widget("FILE_owner_exec").get_active():
            value=value|1
        return "%d" % value
            
    def getGroupAccess(self):
        value=0
        if BaseGUI.wTree.get_widget("FILE_group_read").get_active():
            value=value|4
        if BaseGUI.wTree.get_widget("FILE_group_write").get_active():
            value=value|2
        if BaseGUI.wTree.get_widget("FILE_group_exec").get_active():
            value=value|1
        return "%d" % value
            
    def getAllAccess(self):
        value=0
        if BaseGUI.wTree.get_widget("FILE_all_read").get_active():
            value=value|4
        if BaseGUI.wTree.get_widget("FILE_all_write").get_active():
            value=value|2
        if BaseGUI.wTree.get_widget("FILE_all_exec").get_active():
            value=value|1
        return "%d" % value
            
    def getSetIDBits(self):
        value=0
        if BaseGUI.wTree.get_widget("FILE_owner_setuid").get_active():
            value=value|4
        if BaseGUI.wTree.get_widget("FILE_group_setgid").get_active():
            value=value|2
        if BaseGUI.wTree.get_widget("FILE_all_text").get_active():
            value=value|1
        return "%d" % value
            
    def getData(self):
        retval=BaseGUI.getData(self)
        self.data[1]['accessMode']=[
            "%s%s%s%s" % (self.getSetIDBits(),self.getOwnerAccess(),
                          self.getGroupAccess(),self.getAllAccess())]
        if self.data[1]['fileType'][0]=='python':
            self.data[1]['fileData']=clump.filemaker.compileFile(
                self.data[1]['sourceCode'])
        else:
            self.data[1]['fileData']=self.data[1]['sourceCode']
        return retval

FileGUI.instance=FileGUI()
tempfile.tempdir="/tmp"
