from basegui import *
from clump.interface import *
from clump.obj.node import Node
from gtk import *
from nic   import NICGUI
from mount import MountGUI
from init  import InitGUI
from file  import FileGUI
from delete import confirmDelete

#===========================================================================
# SIGNAL FUNCTIONS: Functions used to handle signals
#
def getSelected(widget_name,objclass,dir,keyfield='cn'):
    widget=BaseGUI.wTree.get_widget(widget_name)
    try:
        name=widget.get_text(widget.selection[0],0)
    except IndexError:
        return None
    objs=getAllInNode(
        objclass,BaseGUI.wTree.get_widget("NODE_name").get_text(),dir)
    data=None
    for obj in objs:
        if obj[1][keyfield][0]==name:
            data=obj
            break
    return data

class NodeSignals:
    def NODE_destroy(obj,event):
        BaseGUI.wTree.get_widget("NODE_window").hide()
        return -1

    def NODE_cancel(obj):
        BaseGUI.wTree.get_widget("NODE_window").hide()

    def NODE_apply(obj):
        NodeGUI.instance.update()

    def NODE_commit(obj):
        if NodeGUI.instance.update():
            BaseGUI.wTree.get_widget("NODE_window").hide()

    def NODE_newnic(obj):
        NICGUI.instance.parent=NodeGUI
        NICGUI.instance.setupNew("ou=NICs,"+NodeGUI.instance.data[0])
        BaseGUI.wTree.get_widget("NIC_window").show()

    def NODE_editnic(obj):
        data=getSelected("NODE_niclist","clumpNIC","NICs","dev")
        if data:
            NICGUI.instance.setup(data)
            BaseGUI.wTree.get_widget("NIC_window").show()
        return

    def NODE_delnic(obj):
        confirmDelete(getSelected("NODE_niclist","clumpNIC","NICs","dev"),
                      NodeGUI)

    def NODE_newmount(obj):
        MountGUI.instance.parent=NodeGUI
        MountGUI.instance.setupNew("ou=Fstab,"+NodeGUI.instance.data[0])
        BaseGUI.wTree.get_widget("MOUNT_window").show()

    def NODE_editmount(obj):
        data=getSelected("NODE_fstablist","clumpMount","Fstab")
        if data:
            MountGUI.instance.setup(data)
            BaseGUI.wTree.get_widget("MOUNT_window").show()
        return

    def NODE_delmount(obj):
        confirmDelete(getSelected("NODE_fstablist","clumpMount","Fstab"),
                      NodeGUI)

    def NODE_newinit(obj):
        InitGUI.instance.parent=NodeGUI
        InitGUI.instance.setupNew("ou=Init,"+NodeGUI.instance.data[0])
        BaseGUI.wTree.get_widget("INIT_window").show()

    def NODE_editinit(obj):
        data=getSelected("NODE_initlist","clumpInitService","Init")
        if data:
            InitGUI.instance.setup(data)
            BaseGUI.wTree.get_widget("INIT_window").show()
        return

    def NODE_delinit(obj):
        confirmDelete(getSelected("NODE_initlist","clumpInitService","Init"),
                      NodeGUI)

    def NODE_newfile(obj):
        FileGUI.instance.parent=NodeGUI
        FileGUI.instance.setupNew("ou=Files,"+NodeGUI.instance.data[0])
        BaseGUI.wTree.get_widget("FILE_window").show()

    def NODE_editfile(obj):
        data=getSelected("NODE_filelist","configFile","Files","fileName")
        if not data:
            data=getSelected("NODE_filelist","configFile","Files")
        if data:
            FileGUI.instance.setup(data)
            BaseGUI.wTree.get_widget("FILE_window").show()
        return

    def NODE_delfile(obj):
        confirmDelete(getSelected("NODE_filelist","configFile","Files"),
                      NodeGUI)

#===========================================================================
# UTILITY FUNCTIONS: Used for filling in tables, combo box lists etc.
#
def getNodeList(attrib):
    list=[]
    nodes=getAllInCluster("clumpNode","Nodes",['objectClass',attrib])
    for node in nodes:
        try:
            if not node[1][attrib][0] in list:
                list.append(node[1][attrib][0])
        except KeyError:
            continue
    list.sort()
    return list

def getNodeNameList():
    return getNodeList('cn')

def getNodeServerList():
    return getNodeList('master')

def getNodeMakeList():
    return getNodeList('make')

def getNodeOwnerList():
    return getNodeList('clumpOwner')

def getNodeGroupList():
    return getNodeList('gid')

def getNodeLocationList():
    return getNodeList('location')

def getNodeGatewayList():
    return getNodeList('ipGatewayNumber')

def getNodeCpuTypeList():
    return getNodeList('cpuType')

#===========================================================================
# GUI Class: Used to fill the GUI window
#
class NodeGUI(BaseGUI):
    instance=None
    yes1,yes2=create_pixmap_from_xpm(
        BaseGUI.wTree.get_widget("NODE_window"),GdkColor(255,255,255),
        "/usr/share/pixmaps/yes.xpm")
    no1,no2=create_pixmap_from_xpm(
        BaseGUI.wTree.get_widget("NODE_window"),GdkColor(255,255,255),
        "/usr/share/pixmaps/no.xpm")
    def __init__(self):
        BaseGUI.__init__(self,"NODE_name",Node,NodeSignals,[
            (["NODE_name"],     "cn",              TextWidget,  {} ),
            (["NODE_enable"],   "enableFlag",      ToggleWidget, {}),
            (["NODE_work"],     "workDir",         TextWidget,  {} ),
            (["NODE_server"],   "masterNode",      ComboWidget,
             { 'list' : getNodeServerList } ),
            (["NODE_cpuspeed"], "cpuSpeed",        TextWidget,  {} ),
            (["NODE_ncpu"],     "nCpu",            TextWidget,  {} ),
            (["NODE_cputype"],  "cpuType",         ComboWidget,
             { 'list' : getNodeCpuTypeList } ),
            (["NODE_memory"],   "memory",          TextWidget,  {} ),
            (["NODE_batch"],    "maxBatchJobs",    TextWidget,  {} ),
            (["NODE_make"],     "make",            ComboWidget,
             { 'list' : getNodeMakeList } ),
            (["NODE_serial"],   "serialNumber",    TextWidget,  {} ),
            (["NODE_model"],    "model",           TextWidget,  {} ),
            (["NODE_owner"],    "clumpOwner",      ComboWidget,
             { 'list' : getNodeOwnerList } ),
            (["NODE_group"],    "gid",             ComboWidget,
             { 'list' : getNodeGroupList } ),
            (["NODE_location"], "location",        ComboWidget,
             { 'list' : getNodeLocationList } ),
            (["NODE_gateway"],  "ipGatewayNumber", ComboWidget,
             { 'list' : getNodeGatewayList } ),
            (["NODE_forwardipv4"],"forwardIPV4",   ToggleWidget, {} )
            ],onNewDisable=["NODE_addnic","NODE_editnic","NODE_delnic",
                            "NODE_addfstab","NODE_editfstab","NODE_delfstab",
                            "NODE_newinit","NODE_editinit","NODE_delinit",
                            "NODE_newfile","NODE_editfile","NODE_delfile",
                            "NODE_newxinetd","NODE_editxinetd","NODE_delxinetd"
                            ])

    def setup(self,data):
        BaseGUI.setup(self,data)
        # Fill in the list of NICs
        try:
            nics=getAllInNode("clumpNIC",data[1]['cn'][0],"NICs",[
                'objectClass','dev','cn','ipHostNumber','onBoot'])
        except KeyError:
            nics=[]
        widget=BaseGUI.wTree.get_widget("NODE_niclist")
        widget.clear()
        for nic in nics:
            widget.append([
                nic[1]['dev'][0],
                nic[1]['cn'][0],
                nic[1]['ipHostNumber'][0],
                nic[1]['onBoot'][0]
                ])
            if string.lower(nic[1]['onBoot'][0]) in [
                "yes", "on",  "1", "true" ]:
                widget.set_pixmap(widget.rows-1,3,NodeGUI.yes1,NodeGUI.yes2)
            else:
                widget.set_pixmap(widget.rows-1,3,NodeGUI.no1,NodeGUI.no2)
        # Fill in the list of Fstab entries
        try:
            entries=getAllInNode("clumpMount",data[1]['cn'][0],"Fstab",[
                'objectClass','cn','mountDirectory','mountType'])
        except KeyError:
            entries=[]
        BaseGUI.wTree.get_widget("NODE_fstablist").clear()
        for entry in entries:
            BaseGUI.wTree.get_widget("NODE_fstablist").append([
                entry[1]['cn'][0],
                entry[1]['mountDirectory'][0],
                entry[1]['mountType'][0]
                ])
        BaseGUI.wTree.get_widget("NODE_fstablist").sort()
        # Fill in the list of init services
        try:
            entries=getAllInNode("clumpInitService",data[1]['cn'][0],
                                 "Init",['cn'])
        except KeyError:
            entries=[]
        widget=BaseGUI.wTree.get_widget("NODE_initlist")
        widget.clear()
        for entry in entries:
            widget.append([entry[1]['cn'][0]])
        widget.sort()
        # Fill in the list of config files
        try:
            entries=getAllInNode("configFile",data[1]['cn'][0],"Files",[
                'cn','fileName','uid','gid','accessMode','fileType'])
        except KeyError:
            entries=[]
        widget=BaseGUI.wTree.get_widget("NODE_filelist")
        widget.clear()
        for entry in entries:
            try:
                widget.append([entry[1]['fileName'][0],
                               entry[1]['uid'][0],
                               entry[1]['gid'][0],
                               entry[1]['accessMode'][0],
                               entry[1]['fileType'][0]
                               ])
            except KeyError:
                widget.append(["%s" % entry[1]['cn'][0],
                               "--","--","--","--"])
        widget.sort()


NodeGUI.instance=NodeGUI()
