from basegui import *
from clump.interface import *
from clump.obj.user import User
from gtk import *
import clump.db

#===========================================================================
# SIGNAL FUNCTIONS: Functions used to handle signals
#
class UserSignals:
    def USER_destroy(obj,event):
        BaseGUI.wTree.get_widget("USER_window").hide()
        return -1

    def USER_cancel(obj):
        BaseGUI.wTree.get_widget("USER_window").hide()

    def USER_apply(obj):
        UserGUI.instance.update()

    def USER_commit(obj):
        if UserGUI.instance.update():
            BaseGUI.wTree.get_widget("USER_window").hide()

    def USER_joingroup(obj):
        widget=BaseGUI.wTree.get_widget("USER_grouplist")
        try:
            widget.set_pixmap(widget.selection[0],0,Pixmap.yes)
        except IndexError:
            return

    def USER_leavegroup(obj):
        widget=BaseGUI.wTree.get_widget("USER_grouplist")
        try:
            widget.set_text(widget.selection[0],0,"")
        except IndexError:
            return


#===========================================================================
# UTILITY FUNCTIONS: Used for filling in tables, combo box lists etc.
#
def getUserList(attrib):
    list=[]
    users=getAllInCluster("clumpUser","Users")
    for user in users:
        try:
            if not user[1][attrib][0] in list:
                list.append(user[1][attrib][0])
        except KeyError:
            continue
    list.sort()
    return list

def getUserVolList():
    return getUserList("homeVolume")

def getUserShellList():
    return getUserList("loginShell")

def getUserInstList():
    return getUserList("ou")

def getUserGroupList():
    list=[]
    groups=getAllInCluster("posixGroup","Groups")
    for group in groups:
        list.append(group[1]['cn'][0])
    list.sort()
    return list

import _gtk

#===========================================================================
# GUI Class: Used to fill the GUI window
#
class UserGUI(BaseGUI):
    instance=None
    def __init__(self):
        BaseGUI.__init__(self,"USER_name",User,UserSignals,[
            (["USER_name"],      "uid",              TextWidget,   {} ),
            (["USER_uid"],       "uidNumber",        TextWidget,   {} ),
            (["USER_homevol"],   "homeVolume",       ComboWidget,
             { 'list' : getUserVolList } ),
            (["USER_home"],      "homeDirectory",    TextWidget,   {} ),
            (["USER_shell"],     "loginShell",       ComboWidget,
             { 'list' : getUserShellList } ),
            (["USER_password"],  "userPassword",     TextWidget,   {} ),
            (["USER_chrname"],   "givenName",        TextWidget,   {} ),
            (["USER_surname"],   "sn",               TextWidget,   {} ),
            (["USER_email"],     "mail",             TextWidget,   {} ),
            (["USER_phone"],     "telephoneNumber",  TextWidget,   {} ),
            (["USER_institute"], "ou",               ComboWidget,
             { 'list' : getUserInstList } ),
            (["USER_group"],     "gidNumber",        ComboWidget,
             { 'list' : getUserGroupList } )
            ])
        UserGUI.instance=self

    def setup(self,data):
        BaseGUI.setup(self,data)
        # Setup group list
        widget=BaseGUI.wTree.get_widget("USER_grouplist")
        widget.clear()
        i=0
        for group in getAllInCluster("posixGroup"):
            widget.append(["",group[1]['cn'][0]])
            try:
                if data[1]['uid'][0] in group[1]['memberUid']:
                    widget.set_pixmap(i,0,Pixmap.yes)
            except KeyError:
                pass
            i=i+1
            try:
                if group[1]['gidNumber'][0]==data[1]['gidNumber'][0]:
                    BaseGUI.wTree.get_widget("USER_group").entry.set_text(
                        group[1]['cn'][0])
            except KeyError:
                BaseGUI.wTree.get_widget("USER_group").entry.set_text("")
        widget.set_sort_column(1)
        widget.sort()

    def getData(self):
        retval=BaseGUI.getData(self)
        for group in getAllInCluster("posixGroup"):
            if self.data[1]['gidNumber'][0]==group[1]['cn'][0]:
                self.data[1]['gidNumber']=group[1]['gidNumber']
        return retval

    def update(self):
        retval=BaseGUI.update(self)
        widget=BaseGUI.wTree.get_widget("USER_grouplist")
        list=[]
        for i in range(0,widget.rows):
            try:
                widget.get_text(i,0)
            except ValueError:
                list.append(widget.get_text(i,1))
        for group in getAllInCluster("posixGroup"):
            try:
                # If the group is in the member list and the UID is not
                # in the member list of the group then we need to add the user
                if group[1]['cn'][0] in list and \
                   not self.data[1]['uid'][0] in group[1]['memberUid']:
                    group[1]['memberUid'].append(self.data[1]['uid'][0])
                    clump.db.db.modObject(group)
                # If the group is not in the member list and the UID is in the
                # member list of the group then we need to remove the user
                elif not group[1]['cn'][0] in list and \
                     self.data[1]['uid'][0] in group[1]['memberUid']:
                    del group[1]['memberUid'][group[1]['memberUid'].index(
                        self.data[1]['uid'][0])]
                    clump.db.db.modObject(group)
            except KeyError:
                pass
        return retval

UserGUI.instance=UserGUI()
