import ldap
import db
import socket
import string

def getCluster(cluster=None):
    if cluster==None:
        cluster=db.db.hostcluster
    try:
        result=db.db.db.search_s("cn=%s,ou=Clusters,%s" % (cluster,db.db.base),
                                ldap.SCOPE_BASE,"objectClass=clumpCluster")
    except:
        return None
    return result[0]

def getNode(host=None):
    if host==None:
        host=string.splitfields(socket.gethostname(),".")[0]
    try:
        result=db.db.db.search_s("cn=%s,ou=Nodes,cn=%s,ou=Clusters,%s" % (
            host,db.db.hostcluster,db.db.base),
                                ldap.SCOPE_BASE,"objectClass=clumpNode")
    except:
        return None
    return result[0]

def getAllInNode(type,host=None,subdir=None,attrlist=None):
    if host==None:
        host=string.splitfields(socket.gethostname(),".")[0]
    if attrlist and not 'objectClass' in attrlist:
        attrlist.append("objectClass")
    dn="cn=%s,ou=Nodes,cn=%s,ou=Clusters,%s" % (
        host,db.db.hostcluster,db.db.base)
    if subdir:
        dn="ou=%s,%s" % (subdir,dn)
    try:
        result=db.db.db.search_s(
            dn,ldap.SCOPE_SUBTREE,"objectClass=%s" % type,attrlist)
    except:
        return None
    filtered=[]
    for res in result:
        if type in res[1]['objectClass']:
            filtered.append(res)
    return filtered

def getNetgroups():
    list=getNetgroupList()
    groups=[]
    for grp in list:
        group=db.db.getObject("cn=%s,ou=Netgroups,cn=%s,ou=Clusters,%s" % (
            grp,db.db.hostcluster,db.db.base))
        if group:
            groups.append(group)
        else:
            print "** ERROR ** Unable to find group '%s' "+\
                  "of which this node is a member!"
    return groups

def getAllInNetgroup(type,group,subdir=None,attrlist=None):
    if attrlist and not 'objectClass' in attrlist:
        attrlist.append("objectClass")
    dn="cn=%s,ou=Netgroups,cn=%s,ou=Clusters,%s" % (
        group,db.db.hostcluster,db.db.base)
    if subdir:
        dn="ou=%s,%s" % (subdir,dn)
    try:
        result=db.db.db.search_s(
            dn,ldap.SCOPE_SUBTREE,"objectClass=%s" % type,attrlist)
    except:
        return None
    filtered=[]
    for res in result:
        if type in res[1]['objectClass']:
            filtered.append(res)
    return filtered

def getAllInNetgroups(type,subdir=None,attrlist=None):
    if attrlist and not 'objectClass' in attrlist:
        attrlist.append("objectClass")
    grps=getNetgroupList()
    result=[]
    for grp in grps:
        dn="cn=%s,ou=Netgroups,cn=%s,ou=Clusters,%s" % (
            grp,db.db.hostcluster,db.db.base)
        if subdir:
            dn="ou=%s,%s" % (subdir,dn)
        try:
            result=result+db.db.db.search_s(
                dn,ldap.SCOPE_SUBTREE,"objectClass=%s" % type,attrlist)
        except:
            pass
    filtered=[]
    for res in result:
        if type in res[1]['objectClass']:
            filtered.append(res)
    return filtered

def getAllInCluster(type,subdir=None,attrlist=None):
    if attrlist and not 'objectClass' in attrlist:
        attrlist.append("objectClass")
    dn="cn=%s,ou=Clusters,%s" % (db.db.hostcluster,db.db.base)
    if subdir:
        dn="ou=%s,%s" % (subdir,dn)
    try:
        result=db.db.db.search_s(
            dn,ldap.SCOPE_SUBTREE,"objectClass=%s" % type,attrlist)
    except ldap.SIZELIMIT_EXCEEDED:
        print "** ERROR ** : Search size limit exceeded, unable to return data"
        return []
    except:
        return []
    filtered=[]
    for res in result:
        if type in res[1]['objectClass']:
            filtered.append(res)
    return filtered

def getNetgroupList():
    host=string.splitfields(socket.gethostname(),".")[0]
    try:
        grps=db.db.db.search_s("ou=Netgroups,cn=%s,ou=Clusters,%s" % (
            db.db.hostcluster,db.db.base),ldap.SCOPE_ONELEVEL,
                               "objectClass=clumpNetgroup")
    except:
        return []
    found=1
    grplist=[]
    while found:
        found=0
        for grp in grps:
            if grp[1]['cn'][0] in grplist: # If we already have the group then
                continue       # continue on to the next one
            match=0            # Flag used to denote a match
            # Loop over the list of groups and see if this group
            # contains any of them. If it does then set the match flag.
            for hostgrp in grplist:
                if grp[1].has_key('memberNisNetgroup') and \
                   hostgrp in grp[1]['memberNisNetgroup']:
                    match=1
                    break
            # Now check to see if the group contains the host directly
            if grp[1].has_key('memberNode') and host in grp[1]['memberNode']:
                match=1
            if match==1:
                grplist.append(grp[1]['cn'][0])
                found=1
    grplist.reverse()
    return grplist

def getServers():
    try:
        servers=[getNode()[1]['masterNode'][0]]
    except KeyError:
        servers=[]
    for ngrp in getNetgroups():
        try:
            servers.append(ngrp[1]['masterNode'][0])
        except KeyError:
            pass
    for srv in getCluster()[1]['masterNode']:
        if not srv in servers:
            servers.append(srv)
    return servers

def getMacros():
    cluster=getCluster()[1]
    node=getNode()[1]
    servers=getServers()
    macros = {
        'hostname' : string.splitfields(socket.gethostname(),".")[0],
        'cluster'  : db.db.hostcluster,
        'domain'   : cluster['domainName'][0],
        'server'   : servers[0],
        'basedn'   : db.db.base,
        'gateway'  : node['ipGatewayNumber'][0]
        }
    macros['serverlist']=""
    macros['serverportlist']=""
    for srv in servers:
        macros['serverlist']=macros['serverlist']+"%s.%s " % (
            srv,cluster['domainName'][0])
        macros['serverportlist']=macros['serverportlist']+"%s.%s:636 " % (
            srv,cluster['domainName'][0])
    try:
        macros['realm']=cluster['krbRealm'][0]
    except:
        macros['realm']=""
    try:
        macros['local_server']=node['masterNode'][0]
    except:
        macros['local_server']=macros['server']
    if getNode()[1]['forwardIPV4'][0]=='1':
        macros['forwardIPV4']='true'
    else:
        macros['forwardIPV4']='false'
    # Overide server value with local one (if provided)
    try:
        macros['server']=getNode()[1]['masterNode'][0]
    except KeyError:
        pass
    return macros
