import string,re
import ldap
from registry import register
from baseobj import BaseObject,Field

class Node(BaseObject):
    def __init__(self):
        self.fields=[
            Field('ipGatewayNumber','gateway',"Network gateway",create=1),
            Field('forwardIPV4','forwardipv4',"Forward IPV4 packets?",must=1),
            Field('nCpu','ncpu',"Number of CPUs",must=1),
            Field('cpuSpeed','cpuSpeed',"Speed in MHz of the CPU(s)",create=1),
            Field('workDir','workdir',"Work or scratch directory",must=1),
            Field('memory','memory',"Size of memory in MB",create=1),
            Field('clumpOwner','owner',"Machine's owner"),
            Field('gid','group',"Machine's user group"),
            Field('location','location',"Location of machine"),
            Field('make','make',"Make of machine"),
            Field('model','model',"Machine's model number"),
            Field('sysAdmin','sysadmin',"System administrators"),
            Field('masterNode','server',"Node's local server"),
            Field('enableFlag','enable',"Is this node enabled?",must=1),
            Field('maxBatchJobs','batch',"Maximum number of batch jobs to run",
                  must=1),
            Field('batchProperty','bprop',"Batch system property")
            ]
        self.objclass=['clumpNode','automount']
        self.subdirs=[
            ('Files',   'configFile'         ),
            ('NICs',    'clumpNIC'           ),
            ('Users',   'clumpUser'          ),
            ('Groups',  'nisGroup'           ),
            ('Init',    'clumpInitService'   ),
            ('Xinetd',  'clumpXinetdService' ),
            ('Fstab',   'clumpMount'         ),
            ('Exports', 'clumpExport'        )
            ]
        BaseObject.__init__(self,"node","clumpNode","cn")

    def onSet(self,dn,data):
        if not 'nCpu' in data.keys():
            data['nCpu']=['1']
        if not 'enableFlag' in data.keys():
            data['enableFlag']=['1']
        if not 'maxBatchJobs' in data.keys():
            data['maxBatchJobs']=data['nCpu']
        if not 'forwardIPV4' in data.keys():
            data['forwardIPV4']=['0']
        if not 'workDir' in data.keys():
            data['workDir']=['/tmp']
        data['automountInformation']="-rw %s:%s" % (
            data['cn'][0],data['workDir'][0])
        return

register(Node())
